<?php
/**
 * @author OEqual - Niravraj Parmar
 * @created-date 10 May 2013
 * @package PDO & pdo-2.1
 *
 * @last-modified Tuesday 30 Devember 2015 06:50 PM TO 06:55 PM
 * @last-modified-by OEqual - Niravraj Parmar
 *
 * @filename pdo-2.1.php
 * @version 2.1
 *
 * pdo-2.1 is a database access library based on PHP's in built driver PDO (PHP Document Object).
 * To use this library need to enable PHP extension php_pdo_mysql. You can also enable it from your php.ini file.
 * Library contain 4 classes, which are required to use this library efficiently.
 * class DB_Error
 * class DB_Recordset_DirectRecordBase
 * class DB_Recordset
 * class DBc
 * Library contain some spacial constants, which are already defined.
 *
 * This version is extended from v1.0 and now it has functionality to deal with direct record.
 * We can have direct record in any format like in array or in object
 */

define('PDO_FETCH_ALL',     'Pdo_Fetch_All');
define('PDO_FETCH_SINGLE',  'Pdo_Fetch_Single');

define('PDO_FETCH_ASSOC',   PDO::FETCH_ASSOC);
define('PDO_FETCH_BOTH',    PDO::FETCH_BOTH);
define('PDO_FETCH_NUM',     PDO::FETCH_NUM);
define('PDO_FETCH_OBJ',     PDO::FETCH_OBJ);
define('PDO_FETCH_LAZY',    PDO::FETCH_LAZY);

define('PDO_S',             'Pdo_Select');
define('PDO_I',             'Pdo_Insert');
define('PDO_U',             'Pdo_Update');
define('PDO_D',             'Pdo_Delete');

/**
 * DB_Error
 * @package pdo-2.1
 */
class DB_Error
{
    # Exception object
    public $PDO_Exception;

    # Current error number and error info
    public $Errno;
    public $Error;
	public $ErrorMsg;
	private $ThrowError   = false;

    public $Debug         = 1;        # Set to 1 for debugging messages.
    public $Start_DTime   = 0;
    public $Stop_DTime    = 0;
    public $Debug_info    = array();  # Store all queries
	public $DieOnHalt     = true;

    # Set false or name of file to write error log. File will be [oe_error_log.txt]
    private $Log_Error    = 'oe';
	# Set class object which handle inserting errors in database.
	public $obj_Log_Error;

    # Set Halt [Yes => halt with message], [No => ignore errors quietly], [Report => ignore errror, but spit a warning]
    public $Halt_On_Error = "Yes";



    /**
     * DB_Error::__construct()
     * @param bool $PDO_Exception = false | object of PDO Exception
     * @return none
     *
     * @__construct set error variables
     */
    /*public function __construct($PDO_Exception = false)
    {
        //$this->set_error($PDO_Exception);
    }*/
    /**
     * DB_Error::Halt_On_Error()
     * @param mixed $flg = [Yes => halt with message], [No => ignore errors quietly], [Report => ignore errror, but spit a warning]
     * @return none
     *
     * @Halt_On_Error set flag for halt error
     */
    public function Halt_On_Error($flg = 'Yes')
    {
	  $this->Halt_On_Error = $flg;
    }
	/**
	 * DB_Error::DieOnHalt()
	 * @param mixed $flg = [true => die on error], [false => continue running rest script]
	 * @return none
	 *
	 * @DieOnHalt set flag for die action
	 */
	public function DieOnHalt($flag = true)
	{
		$this->DieOnHalt = $flag;
	}
	/**
	 * DB_Error::ThrowError()
	 * @param mixed $flg = true | false
	 * @return none
	 *
	 * @ThrowError set flag for throw error action
	 */
	public function ThrowError($flag = false)
	{
		$this->ThrowError = $flag;
	}
    /**
     * DB_Error::Log_Error()
     * @param bool $flag = set error log file name | false
     * @return none
     *
     * @Log_Error set flag to write log file. Only string value will allow to write log file.
     */
    public function Log_Error($flag)
    {
        $this->Log_Error = $flag;
    }
	/**
	 * DB_Error::set_error()
	 * @param mixed $PDO_Exception = object of PDO Exception
	 * @return none
	 *
	 * @set_error set error variables
	 */
	public function set_error($PDO_Exception, $db_error = false, $query = false, $value = false)
	{
		if (is_object($PDO_Exception))
		{
			$this->PDO_Exception=   $PDO_Exception;
			$this->Error        =   $this->PDO_Exception->getMessage();
			$this->Errno        =   $this->PDO_Exception->getCode();

			$this->ErrorMsg['DateTime']         =   date('Y-m-d H:i:s');
			$this->ErrorMsg['ProgramError']     =   $this->Error;
			$this->ErrorMsg['DatabaseError']    =   $db_error;
			$this->ErrorMsg['SQLQuery']         =   $query;
			$this->ErrorMsg['SQLValue']         =   print_r($value,true);
			$this->ErrorMsg['HaltedReport']     =   "Session Halted";
		}
	}
	/**
	 * DB_Error::Write_Log()
	 * @param bool|string $db_error = false | string messge
	 * @param bool|string $query = false | sql query string
	 * @param array|bool $value = false | array of values
	 * @return none
	 *
	 * @Write_Log write database error in given log file
	 */
    public function Write_Log()
    {
        if(is_string($this->Log_Error))
        {
            $filename = dirname(__FILE__). '/'.$this->Log_Error.'_error_log.txt';
	        $handle = fopen($filename, 'a');

	        $log = "\n\n".print_r($this->ErrorMsg,true);

        	fwrite($handle, $log);
            fclose($handle);
        }

	    if(is_object($this->obj_Log_Error) && method_exists($this->obj_Log_Error,'_DBErrorLog'))
	    {
		    $this->obj_Log_Error->_DBErrorLog($this->ErrorMsg);
	    }
    }

	/**
	 * DB_Error::halt_html_msg()
	 * @param bool|string $db_error = false | string message
	 * @param bool|string $query = false | sql query string
	 * @param array|bool $value = false | array of values
	 * @return html error message
	 *
	 * @halt_html_msg generate html error message to display
	 */
    private function halt_html_msg($db_error = false, $query = false, $value = false)
    {
        if ($this->Halt_On_Error != "No")
        {
            echo    '<style type="text/css">
                        div.db-error    {font-family:arial;font-size:12px;background-color:#FFEAF5;border:1px solid #800000;margin:5px;padding:5px;}
                        div.db-error h4 {font-size:15px;margin:0 0 10px 0;background-color:#D95762;color:#FDFD89;padding:0 2px;}
                    </style>';

            if (!empty($this->Error))
                echo '<div class="db-error"><h4>Program Error</h4>'.$this->Error.'</div>';

            if ($db_error != false)
                echo '<div class="db-error"><h4>Database Error</h4>'.$db_error.'</div>';

            if ($query != false)
                echo '<div class="db-error"><h4>SQL Query</h4>'.$query.'</div>';

            if ($query != false && is_array($value))
                echo '<div class="db-error"><h4>SQL Value</h4><pre>'; print_r($value); echo '</pre></div>';
        }
    }

	/**
	 * DB_Error::halt()
	 * @param bool|object $PDO_Exception = false  | PDO Exception object
	 * @param bool|string $db_error = false | string messge
	 * @param bool|string $query = false | sql query string
	 * @param array|bool $value = false | array of values
	 * @return error
	 *
	 * @halt displying error as per conditional flags
	 */
    public function halt($PDO_Exception = false, $db_error = false, $query = false, $value = false)
    {
	    # Set error details
        $this->set_error($PDO_Exception, $db_error, $query, $value);

        # Write error log in text file
        $this->Write_Log();

        # Throw exception
	    if($this->ThrowError === true)
	        throw new Exception(utf8_encode(serialize($this->ErrorMsg)));
	    # Display html error message
	    else
		    $this->halt_html_msg($db_error, $query, $value);

	    if($this->DieOnHalt == true)
            exit('<div class="db-error">Session Halted.</div>');
    }
    /**
     * DB_Error::start_debug()
     * @return none
     *
     * @start_debug set start debug time
     */
    public function start_debug()
    {
        if ($this->Debug)
        {
            # Store start time
            list($usec, $sec) = explode(" ", microtime());
            $this->Start_DTime = ((float)$usec + (float)$sec);
        }
    }
    /**
     * DB_Error::stop_debug()
     * @param mixed $Query_String = sql query sring
     * @param mixed $Value = false | array of values to execute
     * @return none
     *
     * @stop_debug check for stop time and set debug info
     */
    public function stop_debug($Query_String, $Value = false)
    {
	    global $Debug_info;

	    if ($this->Debug)
        {
	        # Check end time
            list($usec, $sec) = explode(" ", microtime());
            $this->Stop_DTime = ((float)$usec + (float)$sec);

	        $Debug_info[] = $this->Debug_info[] =
								array(  'Query'		=>	$Query_String,
                                        'Value'     =>  $Value,
            							'Time'		=>	microtime(),
            							'ExecTime'	=>	($this->Stop_DTime - $this->Start_DTime),
            							'QueryTime'	=>	date('Y-M-d H:i:s'),
                                    );
        }
    }
}
/**
 * DB_Recordset_DirectRecordBase
 * @package pdo-2.1
 */
final class DB_Recordset_DirectRecordBase
{
	# result array and current row number
	public $Record      = '';

	/**
	 * DB_Recordset_ArrayBase::f()
	 * @param array/object $record = Entire record in array or in object form
	 * @return none
	 *
	 * @set_record set current record in class variable
	 */
	public function set_record($record)
	{
		$this->Record = $record;
	}
	/**
	 * DB_Recordset_ArrayBase::f()
	 * @param mixed $Name = name of field in record
	 * @return value of field
	 *
	 * @f check for given field value and return that value
	 */
	public function f($Name)
	{
		return (is_array($this->Record) && isset($this->Record[$Name])?$this->Record[$Name]:(is_object($this->Record) && isset($this->Record->$Name)?$this->Record->$Name:false));
	}
}
/**
 * DB_Recordset
 * @package pdo-2.1
 */
final class DB_Recordset extends DB_Error
{
    # configuration parameters
    public $Auto_Free     = 0;     # Set to 1 for automatic free result()

    # result array and current row number
    public $Record      = '';
    public $Row		    = 0;
    public $TotalRow	= 0;

    # link and query handles
    private $Link_ID  = 0;
    private $Query_ID = 0;
    private $Query;

    /**
     * DB_Recordset::__construct()
     * @param mixed $connection = DB connection
     * @param mixed $result = eexecuted query result
     * @param bool $Query = sql query string
     * @return object of class
     *
     * @__construct set required variables
     */
    public function __construct($connection, $result, $Query = false)
    {
        # Set file name to log error in text file
        $this->Log_Error('Recordset');

        $this->Debug        =  isset($_SESSION['SHOW_QUERY'])?1:0;
        $this->Link_ID 		=  $connection;
        $this->Query_ID 	=  $result;
        $this->Query        =  $Query;
        $this->TotalRow 	=  $this->num_rows();
    }
    /**
     * DB_Recordset::query_id()
     * @return executed query result
     */
    public function query_id()
    {
        return $this->Query_ID;
    }
    /**
     * DB_Recordset::free_result()
     * @return none
     *
     * @free_result frees up the connection to the server
     */
    public function free_result()
    {
        $this->Query_ID->closeCursor();
        //$this->Query_ID = 0;
    }
    /**
     * DB_Recordset::num_rows()
     * @return number of rows
     *
     * @num_rows get number of selected rows by sql query
     */
    public function num_rows()
    {
    	$rows = $this->Query_ID->rowCount();
    	return $rows;
    }
    /**
     * DB_Recordset::num_fields()
     * @return int number of field(s)
     *
     * @num_fields get number of selected fields by sql query
     */
    public function num_fields()
    {
        $fields = $this->Query_ID->columnCount();
    	return $fields;
    }
    /**
     * DB_Recordset::fetch_record()
     * @param mixed $fetch = PDO_FETCH_ALL | PDO_FETCH_SINGLE
     * @param $result_type = PDO_FETCH_ASSOC | PDO_FETCH_BOTH | PDO_FETCH_NUM | PDO_FETCH_OBJ | PDO_FETCH_LAZY
     * @param $key = database field name. To set field value as a key in return data. Will work only with [$fetch = PDO_FETCH_ALL] argument
     * @return record object
     *
     * @fetch_record fetch all data in given format
     */
    public function fetch_record($fetch = PDO_FETCH_ALL, $result_type = PDO_FETCH_ASSOC, $key = false)
    {
        $data = '';

        # Get data from database
        if ($fetch == PDO_FETCH_ALL && $result_type != PDO_FETCH_LAZY)
        {
            if (is_string($key))
            {
                if ($result_type == PDO_FETCH_OBJ)
                {
                    while ($_data = $this->Query_ID->fetch($result_type))
                        $data->{$_data->$key} = $_data;
                }
                else
                {
                    while ($_data = $this->Query_ID->fetch($result_type))
                        $data[$_data[$key]] =   $_data;
                }
            }
            else
                $data = $this->Query_ID->fetchAll($result_type);
        }
        elseif ($fetch == PDO_FETCH_SINGLE || $result_type == PDO_FETCH_LAZY)
            $data = $this->Query_ID->fetch($result_type);

        return $data;
    }
    /**
     * DB_Recordset::next_record()
     * @return array of data
     *
     * @next_record get record from executed query and return in array format
     */
    public function next_record($fetch = PDO_FETCH_SINGLE, $result_type = PDO_FETCH_ASSOC, $key = false)
    {
        try
        {
            $this->Record = $this->fetch_record($fetch, $result_type, $key);

            if ($this->Record)
                $this->Row   += 1;

	        $record = (is_array($this->Record)?is_array($this->Record):(is_object($this->Record)?is_object($this->Record):false));

            if ($record != false && $this->Auto_Free)
            {
                $this->free_result();
            }

            return $record;
        }
        catch (PDOException $e)
        {
            $this->halt($e,"next_record() called with no query pending.");
        }
    }/**
	 * DB_Recordset::set_record()
	 * @param array/object $record = Entire record in array or in object form
	 * @return none
	 *
	 * @set_record set current record in class variable
	 */
	public function set_record($record)
	{
		$this->Record = $record;
	}
    /**
     * DB_Recordset::f()
     * @param mixed $Name = name of field in record
     * @return value of field
     *
     * @f check for given field value and return that value
     */
    public function f($Name)
    {
	    return (is_array($this->Record) && isset($this->Record[$Name])?$this->Record[$Name]:(is_object($this->Record) && isset($this->Record->$Name)?$this->Record->$Name:false));
    }
    /**
     * DB_Recordset::p()
     * @param mixed $Name = name of field in record
     * @return print value of field
     *
     * @p check for given field value and print that value
     */
    public function p($Name)
    {
	    echo (is_array($this->Record) && isset($this->Record[$Name])?$this->Record[$Name]:(is_object($this->Record) && isset($this->Record->$Name)?$this->Record->$Name:false));
    }
    /**
     * DB_Recordset::columnMeta()
     * @param mixed $id = index number for field
     * @param $meta = name | native_type | flags | table | len | precision | pdo_type
     * @return meta data
     *
     * @columnMeta get meta data of field for given index
     */
    public function columnMeta($id, $meta = 'name')
    {
        if (isset($this->Query_ID))
        {
            $meta_data = $this->Query_ID->getColumnMeta(0);
            return $meta_data[$meta];
        }
    }
}

/**
 * DBc
 * @package PDO / pdo-2.1
 */
final class DBc extends DB_Error
{
    # connection parameters
    private $Driver;
    private $Host;
    private $User;
    private $Password;
    private $Database;
    private $Charset;
    private $Port;
    private $Persistency;
    private $Driver_Option = array();

    # connection and query handles
    private $Link_ID  = 0;
    private $Sth_Result;

    # Store class object
    public static $obj;

    /**
     * DBc::unset_instance()
     * @return none
     *
     * @instance unset class instance stored in static variable $obj
     */
    public static function unset_instance()
    {
        self::$obj = null;
    }

    /**
     * DBc::__construct()
     * @param string $Driver = name of driver to use for PDO connection
     * @param mixed $Host = Host name
     * @param mixed $User = DB Username
     * @param mixed $Password = DB Passeword
     * @param mixed $Database = Database name
     * @param string $Charset = utf8| any charset
     * @param int $Port = 3306 | port number for mysql server
     * @param bool $Persistency = true | false
     * @param array $Driver_Option = attribute to set
     * @return connection
     *
     * @__construct set required connection parameters and connect to database
     */
    public function __construct($Host,$User,$Password,$Database = false,$Driver='mysql',$Charset = 'utf8',$Port = 3306,$Persistency = false,$Driver_Option = array())
    {
	    # Set file name to log error in text file
        $this->Log_Error('DataBase');

        $this->Debug           = isset($_SESSION['SHOW_QUERY'])?1:0;
        $this->Driver          = $Driver;
        $this->Host            = $Host;
        $this->User            = $User;
    	$this->Password        = $Password;
        $this->Database 	   = $Database;
    	$this->Charset         = $Charset;
    	$this->Port            = $Port;
        $this->Persistency	   = $Persistency;

        $Driver_Option         =    array_merge($Driver_Option,
                                                array(  PDO::ATTR_EMULATE_PREPARES      =>  false,
                                                        PDO::ATTR_ERRMODE               =>  PDO::ERRMODE_EXCEPTION,
                                                        PDO::ATTR_CASE                  =>  PDO::CASE_NATURAL,
                                                        PDO::ATTR_DEFAULT_FETCH_MODE    =>  PDO_FETCH_ASSOC,
                                                        PDO::MYSQL_ATTR_INIT_COMMAND    => "SET NAMES utf8",
                                                        PDO::MYSQL_ATTR_LOCAL_INFILE    =>  true,
                                                        PDO::ATTR_PERSISTENT            =>  true
                                                    )
                                                );
        if ($this->Persistency == true)
            $Driver_Option = array_merge($Driver_Option,array(PDO::ATTR_PERSISTENT => true,));

        $this->Driver_Option   =  $Driver_Option;

        $this->db_connect();
    }
	public function __sleep()
	{
		return array();
	}
	public function __wakeup()
	{
		$this->db_connect();
	}
	public function __toString()
	{
		return "You are at ".$this->Database;
	}
    /**
     * DBc::db_connect()
     * @return connection
     *
     * @db_connect check for database connection
     */
    public function db_connect()
    {
        # Establish connection if not connected and select database
        if ($this->Link_ID == 0)
        {
            try
            {
                # create PDO object
                $this->Link_ID = new PDO(   $this->Driver.':host='.
                                            $this->Host.';'.
                                            (!empty($this->Database)?'dbname='.$this->Database.';':'').
                                            'port='.$this->Port.','.
                                            'charset='.$this->Charset,
                                            $this->User,
                                            $this->Password,
                                            $this->Driver_Option
                                        );
            }
            catch(PDOException $e)
            {
	            $this->halt($e,"Database connection failed.");
            }
        }

        return $this->Link_ID;
    }
    /**
     * DBc::db_close()
     * @return true
     *
     * @db_close close PDO open connection to database
     */
    public function db_close()
    {
        if($this->Link_ID)
        {
        	$this->Link_ID = null;
            //unset($this);
            return true;
        }
    }
    /**
     * DBc::get_link_id()
     * @return connection link
     */
    public function get_link_id()
    {
        return $this->Link_ID;
    }
    /**
     * DBc::get_sth_result()
     * @return current result resource
     */
    public function get_sth_result()
    {
        return $this->Sth_Result;
    }
    /**
     * DBc::query_debug()
     * @return array Debug Info
     */
    public function query_debug()
    {
        return $this->Debug_info;
    }
    /**
     * DBc::database_name()
     * @return Database name
     */
    public function database_name()
    {
        return $this->Database;
    }
    /**
     * DBc::q_exec()
     * @param mixed $Query_String = sql query string
     * @return number of affected rows
     *
     * @q_exec execute sql statement
     */
    public function q_exec($Query_String)
    {
        $q_exec = $this->Link_ID->exec($Query_String);
        return $q_exec;
    }
    /**
     * DBc::q_query()
     * @param mixed $Query_String = sql query string
     * @return run query
     *
     * @q_query execute sql statement
     */
    public function q_query($Query_String)
    {
        $q_query = $this->Link_ID->query($Query_String);
        return $q_query;
    }
    /**
     * DBc::q_prepare()
     * @param mixed $Query_String = sql query string
     * @return prepared query
     *
     * @q_prepare prepare sql statement
     */
    public function q_prepare($Query_String)
    {
        $this->Sth_Result = $this->Link_ID->prepare($Query_String);
        return $this->Sth_Result;
    }
    /**
     * DBc::q_execute()
     * @param bool $Value = false | array of values | array(key => value)
     * @return executed sql query
     *
     * @q_execute execute prepared sql statement
     */
    public function q_execute($Value = false)
    {
        if (is_array($Value) && count($Value) > 0)
            $this->Sth_Result->execute($Value);
        else
            $this->Sth_Result->execute();
    }
    /**
     * DBc::start_transaction()
     * @return none
     *
     * @start_transaction  Initiates a transaction and turns off autocommit mode.
     */
    public function start_transaction()
    {
        $this->Link_ID->beginTransaction();
    }
    /**
     * DBc::end_transaction()
     * @return none
     *
     * @end_transaction  commits a transaction.
     */
    public function end_transaction()
    {
        $this->Link_ID->commit();
    }
	/**
	 * DBc::in_transaction()
	 * @return true | false - check whether transaction is on or not
	 *
	 * @end_transaction  commits a transaction.
	 */
	public function in_transaction()
	{
		return $this->Link_ID->inTransaction();
	}
    /**
     * DBc::stop_transaction()
     * @return none
     *
     * @stop_transaction  rolls back a transaction.
     */
    public function stop_transaction()
    {
        $this->Link_ID->rollBack();
    }
    /**
     * DBc::run_query()
     * @param string $Query_String = SQL query string
     * @param array $Value = false | array(key => value)
     * @param string $Action = flase | PDO_S (SELECT) | PDO_I (INSERT) | PDO_U (UPDATE) | PDO_D (DELETE)
     * @param bool $Is_Unbuffered = false | true
     * @return void
     *
     * @run_query run sql query to database
     */
    public function run_query($Query_String, $Value = false, $Action = PDO_S, $Is_Unbuffered = false)
    {
        try
        {
            # start debug
            $this->start_debug();

            # start transaction
            if ($Action != PDO_S && $this->in_transaction() === false)
            {
	            $t_started = true;
	            $this->start_transaction();
            }

            if ($Action === false)
            {
                # Run sql statement
                $this->q_query($Query_String);

                $return = true;
            }
            else
            {
                # Prepare sql statement
                $this->q_prepare($Query_String);

                # Execute sql statement with or without values
                $this->q_execute($Value);

                if($Action == PDO_S)
                {
            		# return Recordset
            		$return = new DB_Recordset($this->Link_ID, $this->Sth_Result, $Query_String);
                }
                elseif ($Action == PDO_I)
                {
                    $return = $this->last_inserted_id();
                }
                elseif ($Action == PDO_U || $Action == PDO_D)
                {
                    $return = $this->affected_rows();
                }
                else
                {
                    $return = true;
                }
            }

            # commit changes
            if ($Action != PDO_S && isset($t_started) && $t_started === true)
                $this->end_transaction();

            # stop debug
            $this->stop_debug($Query_String, $Value);

            return $return;
        }
        catch (PDOException $e)
        {
            # roll back all changes
            if ($Action != PDO_S)
                $this->stop_transaction();

             $this->halt($e,"Invalid SQL query.",$Query_String,$Value);
        }
    }
	public function getRecordCount($table_name)
	{
		try
		{
			if (is_array($table_name))
				$table_name = implode(',',$table_name);

			$query = "SELECT COUNT(*) AS count FROM ". $table_name;

			# start debug
			$rs = $this->run_query($query);
			$arr= $rs->fetch_record(PDO_FETCH_SINGLE,PDO_FETCH_ASSOC);

			return $arr['count'];
		}
		catch (PDOException $e)
		{
			$this->halt($e,"Unable get total count for table.");
		}
	}
    /**
     * DBc::use_db()
     * @param string $db_name = Name of database to use
     * @return none
     *
     * @use_db uses the given database and make it available to run queries
     */
    public function use_db($db_name)
    {
        try
        {
            $Query_String =  "USE ".$db_name;

            # Run sql statement
            $this->q_query($Query_String);

            # Set new database name
            $this->Database =   $db_name;

            return true;
        }
        catch (PDOException $e)
        {
            $this->halt($e,"Can not use database.",$Query_String);
        }
    }
    /**
     * DBc::last_inserted_id()
     * @return last insert id | false
     *
     * @last_inserted_id get primary key id for last inserted record
     */
    public function last_inserted_id()
    {
        try
        {
        	$id = $this->Link_ID->lastInsertId();
        	return $id;
        }
        catch (PDOException $e)
        {
        	$this->halt($e,"Unable to get last insert id.");
        }
    }
    /**
     * DBc::affected_rows()
     * @return number of rows
     *
     * @affected_rows get number of rows affected by sql query
     */
    public function affected_rows()
    {
        try
        {
        	$rows = $this->Sth_Result->rowCount();
        	return $rows;
        }
        catch (PDOException $e)
        {
        	$this->halt($e,"Unable to get number of affected row(s).");
        }
    }
    /**
     * DBc::lock_tables()
     * @param mixed $table = table name | array(mode => table name)
     * @param string $mode
     * @return none
     *
     * @lock_tables lock given tables for given mode
     */
    public function lock_tables($table, $mode = "write")
    {
        try
        {
            $query = "LOCK TABLES ";
            if (is_array($table))
            {
                while (list($key,$value) = each($table))
                {
                    if (!is_int($key))
                    {
                        // texts key are "read", "read local", "write", "low priority write"
                        $query .= "$value $key,";
                    }
                    else
                    {
                        $query .= "$value $mode,";
                    }
                }
                $query = rtrim($query,',');

            }
            else
            {
                $query .= "$table $mode";
            }

            # start debug
            $this->start_debug();

            $lock = $this->q_exec($query);

            # stop debug
            $this->stop_debug($query);

            return true;
        }
        catch (PDOException $e)
        {
            $this->halt($e,"Unable to lock table(s).");
        }
    }
    /**
     * DBc::unlock_tables()
     * @return none
     *
     * @unlock_tables unlock tables
     */
    public function unlock_tables()
    {
        try
        {
            $query = "UNLOCK TABLES";

            # start debug
            $this->start_debug();

            $unlock = $this->q_exec($query);

            # stop debug
            $this->stop_debug($query);

            return true;
        }
        catch (PDOException $e)
        {
            $this->halt($e,"Unable to unlock table(s).");
        }
    }
    /**
     * DBc::show_tables()
     * @return array of table name
     *
     * @show_tables get all table names and return as an array
     */
    public function show_tables()
    {
        try
        {
            $query = "SHOW TABLES";
            $this->q_prepare($query);
            $this->q_execute();
            $all_tables = array();

            while ($table = $this->Sth_Result->fetch())
                $all_tables[] = $table[0];

            return $all_tables;
        }
        catch (PDOException $e)
        {
            $this->halt($e,"Unable to get table(s).");
        }
    }
    /**
     * DBc::truncate_table()
     * @param mixed $table_name = name of table | array of tables
     * @return none
     *
     * @truncate_table will empty all given tables from database
     */
    public function truncate_table($table_name)
    {
        try
        {
	        $total_records = 0;
            if (is_array($table_name))
                $table_name = implode(',',$table_name);
			else
	            $total_records = $this->getRecordCount($table_name);

            $query = "TRUNCATE ". $table_name;

            # start debug
            $this->start_debug();

            $this->q_prepare($query);
            $this->q_execute();

	        $return = $this->affected_rows();

            # stop debug
            $this->stop_debug($query);

            return $total_records;
        }
        catch (PDOException $e)
        {
            $this->halt($e,"Unable to truncate table(s).");
        }
    }
    /**
     * DBc::drop_table()
     * @param mixed $table_name = name of table | array of tables
     * @return none
     *
     * @drop_table will drop all given tables from database
     */
    public function drop_table($table_name)
    {
        try
        {
            if (is_array($table_name))
                $table_name = implode(',',$table_name);

            $query = "DROP TABLE IF EXISTS ". $table_name;

            # start debug
            $this->start_debug();

            $this->q_prepare($query);
            $this->q_execute();

            # stop debug
            $this->stop_debug($query);
        }
        catch (PDOException $e)
        {
            $this->halt($e,"Unable to drop table(s).");
        }
    }
	public function create_database($db_name)
	{
		try
		{
			$query = "CREATE DATABASE IF NOT EXISTS ".$db_name." DEFAULT CHARACTER SET ".$this->Charset." COLLATE utf8_general_ci";

			# start debug
			$this->start_debug();

			$result = $this->q_query($query);

			# stop debug
			$this->stop_debug($query);

			return true;
		}
		catch (PDOException $e)
		{
			$this->halt($e,"Unable to create database.");
		}
	}
    /**
     * DBc::drop_database()
     * @return none
     *
     * @drop_database drop current database
     */
    public function drop_database()
    {
        try
        {
            $query = "DROP DATABASE ". $this->Database;

            # start debug
            $this->start_debug();

            $result = $this->q_query($query);

            # stop debug
            $this->stop_debug($query);
        }
        catch (PDOException $e)
        {
            $this->halt($e,"Unable to drop database.");
        }
    }
	/**
	 * DBc::show_cur_user_privileges()
	 * @return object of record set | array of privileges
	 *
	 * @show_cur_user_privileges get current user's granted privileges in database
	 */
	public function show_cur_user_privileges($obj=true)
	{
		try
		{
			$query = "SHOW PRIVILEGES";

			# start debug
			$this->start_debug();

			$rs = $this->run_query($query);

			# stop debug
			$this->stop_debug($query);

			if($obj === false)
			{
				$arr= $rs->fetch_record(PDO_FETCH_ALL, PDO_FETCH_ASSOC, 'Privilege');
				return $arr;
			}
			else
				return $rs;
		}
		catch (PDOException $e)
		{
			$this->halt($e,"Unable get current user's privileges.");
		}
	}
	/**
	 * DBc::import_by_file()
	 * @param string $file = file name with full path (i.e. ../db_folder/db_file.sql)
	 * @return true on success| false on failure
	 *
	 * @import_by_file will try to import database by checking functions and if possible import database
	 */
	public function import_by_file($file)
	{
		$command='mysql -h '.$this->Host.' -u '.$this->User.' -p'.$this->Password.' '.$this->Database .' < ' .$file;

		if(function_exists('exec'))
			exec($command,$output,$status);
		elseif(function_exists('system'))
			system($command,$status);
		elseif(function_exists('passthru'))
			passthru($command,$status);
		else
			return false;

		if($status === 0)
			return true;
		else
			return false;
	}
	/**
	 * DBc::import_by_sql()
	 * @param string $file = file name with full path. this file should contain written SQL queries separated by semicolon (;)  (i.e. ../db_folder/db_file.sql)
	 * @param array $replace_code = false | array of codes to replace
	 * @return true on success| false on failure
	 *
	 * @import_by_sql will try to import database by sql query
	 */
	public function import_by_sql($file, $replace_code=false)
	{
		$file_content   =   file_get_contents($file);

		if(is_array($replace_code))
		{
			foreach ($replace_code as $code => $value)
			{
				$file_content   =   str_replace($code,$value,$file_content);
			}
		}


		try
		{
			$this->run_query($file_content,false,false);
			return true;
		}
		catch(PDOException $e)
		{
			try
			{
				$query = explode(';',$file_content);
				foreach ($query as $key=>$value)
				{
					$value = trim($value);
					if(!empty($value))
						$this->run_query($value,false,false);
				}
			}
			catch(PDOException $e)
			{
				$this->halt($e,"Invalid SQL query.",$file_content);
			}
		}
	}
	public function import_by_xml(){}
    /**
	 * DBc::set_group_concat_max_len()
	 * @param int $len = integer more then 0
	 * @return none
	 *
	 * @set_group_concat_max_len will try to increase database group concat max len by given number (1024 * $len)
     * Assigned value to $len will be multiply by 1024
	 */
    public function set_group_concat_max_len($len=2)
    {
        $len = (int) $len;
        $query = "SET group_concat_max_len = (1024*".$len.")";
        $this->q_query($query);
    }
    /**
     * DBc::__destruct()
     * @return none
     *
     * @__destruct unset connection
     */
    public function __destruct()
    {
	    $this->db_close();
    }
}
?>